# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import shutil
import os
import sys
from importlib.metadata import version


sys.path.insert(0, os.path.abspath("../../"))
# -- Project information -----------------------------------------------------


project = "pyPLNmodels"
copyright = "2023, Bastien Batardière, Julien Chiquet, Joon Kwon"  # pylint: disable=redefined-builtin
author = "Bastien Batardière, Julien Chiquet, Joon Kwon"

# The full version, including alpha/beta/rc tags
release = version(project)
version = release

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx_rtd_theme",
    "sphinx.ext.autosectionlabel",
    "sphinx.ext.viewcode",
    "sphinx_copybutton",
    "sphinx_codeautolink",
    "autoclasstoc",
]
# source_suffix = [".rst", ".md"]
napoleon_google_docstring = False
napoleon_numpy_docstring = True


# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]
html_theme = "sphinx_rtd_theme"


## My config
autodoc_typehints = "description"
autodoc_member_order = "bysource"
autodoc_default_options = {
    "members": True,
    "special-members": "__init__",
    "private-members": False,
    "inherited-members": True,
    "undoc-members": False,
}
# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]
# html_extra_path = ['vignettes']
html_title = "pyPLNmodels"


def copy_html_vignettes(app, exception):  # pylint: disable=unused-argument
    """Copy only HTML files from vignettes/ to the build directory."""
    source = os.path.join(app.srcdir, "vignettes")
    destination = os.path.join(app.outdir, "vignettes")

    if os.path.exists(source):
        os.makedirs(destination, exist_ok=True)  # Ensure the target folder exists
        for file in os.listdir(source):
            if file.endswith(".html"):  # Only copy .html files
                shutil.copy(os.path.join(source, file), os.path.join(destination, file))


def setup(app):
    """
    Add the copying of html vignettes after the build.
    """
    app.connect("build-finished", copy_html_vignettes)
